@@ -420,10 +420,8 @@ static int SaveChamp(char *plot)
 #	include "types.h"
 #else /* not macintosh */
 #       ifndef VMS
-#	ifndef __ABSC__
 #   	include <sys/types.h>	/* for <netinet/in.h> on some systems */
-#	endif
-#   	if (!defined __MSC__) && !(defined __ABSC__) && !(defined __MINGW32__) 
+#   	if (!defined __MSC__) && !(defined __MINGW32__) 
 #          include <netinet/in.h>	/* for htonl() */
 #   	endif
 #	endif
