@@ -15,6 +15,14 @@
 
 #include "PloEch.h"
 
+#if WIN32
+#include "../os_specific/win_mem_alloc.h" /* MALLOC */
+#else
+#include "../os_specific/sci_mem_alloc.h" /* MALLOC */
+#endif
+
+extern void updateScaleIfRequired( sciPointObj * pSubWin ) ;
+
 extern BOOL update_specification_bounds(sciPointObj *psubwin, double *rect,int flag);
 
 int plot2dn(integer ptype,char *logflags,double *x,double *y,integer *n1,integer *n2,integer *style,char *strflag,char *legend,double *brect,integer *aaint, BOOL flagNax, integer lstr1,integer lstr2);
@@ -26,25 +34,30 @@ int plot2dn(integer ptype,char *logflags,double *x,double *y,integer *n1,integer
  * ensuite il reste qu'appeler la fonction du dessin de l'objet 
  *-----------------------------------------------*/
 
-void Objrect (x,y,width,height,fillflag,fillcolor,n,hdl,flagstring)
-    double *x,*y,*width,*height;
-    int fillflag, fillcolor,n;
-    long *hdl;
-    BOOL flagstring;
+void Objrect (x,y,width,height,foreground,background,isfilled,isline,n,hdl,flagstring)
+     double *x,*y,*width,*height;
+     int *foreground, *background;
+     BOOL isfilled, isline;
+     int n;
+     long *hdl;
+     BOOL flagstring;
 { 
   sciPointObj *psubwin;
+  sciPointObj * newObj = NULL ;
   psubwin = (sciPointObj *)sciGetSelectedSubWin (sciGetCurrentFigure ());
   
-  sciSetCurrentObj (ConstructRectangle
-		    (psubwin ,*x,*y,*height, *width, 0, 0,fillflag, fillcolor ,n,flagstring));
+  newObj = ConstructRectangle( psubwin ,*x,*y,*height, *width, 0, 0,
+                               foreground, background, isfilled, isline, n, flagstring ) ;
+  
+  if ( newObj == NULL )
+  {
+    /* an error occured */
+    *hdl = -1 ;
+    return ;
+  }
+  sciSetCurrentObj( newObj ) ;
   
   *hdl=sciGetHandle(sciGetCurrentObj ()); 
-  if (pSUBWIN_FEATURE(psubwin)->surfcounter>0) {
-    Merge3d(psubwin); /* an addtomerge function should be much more efficient */
-    /*     EraseAndOrRedraw(sciGetSelectedSubWin (sciGetCurrentFigure ()));} */ /* inhibit EraseAndOrRedraw for now F.Leray 20.12.04 */
-    sciDrawObj(sciGetCurrentFigure ());}
-  else
-    sciDrawObjIfRequired(sciGetCurrentObj ());
 }
 
 
@@ -52,10 +65,11 @@ void Objrect (x,y,width,height,fillflag,fillcolor,n,hdl,flagstring)
  * Objarc : 
  *-----------------------------------------------*/
 
-void Objarc (angle1,angle2,x,y,width,height,color,fill,hdl)
+void Objarc (angle1,angle2,x,y,width,height,foreground,background,isfilled,isline,hdl)
     int *angle1,*angle2;
     double *x,*y,*width,*height;
-    int color,fill;
+    int *foreground, *background;
+    BOOL isfilled,isline;
     long *hdl;
 { 
   sciPointObj *psubwin, *pobj;
@@ -63,7 +77,7 @@ void Objarc (angle1,angle2,x,y,width,height,color,fill,hdl)
 
   sciSetCurrentObj (ConstructArc
          (psubwin,*x,*y,
-	  *height, *width, *angle1, *angle2, color, fill));
+	  *height, *width, *angle1, *angle2, foreground, background, isfilled, isline));
   pobj = sciGetCurrentObj();
  
   *hdl=sciGetHandle(pobj);
@@ -83,31 +97,21 @@ void Objpoly (x,y,n,closed,mark,hdl)
 { 
   sciPointObj *psubwin, *pobj;
   psubwin = (sciPointObj *)sciGetSelectedSubWin (sciGetCurrentFigure ());
-
-  sciSetCurrentObj (ConstructPolyline(psubwin,x,y,PD0,closed,n,1,1)); 
-  pobj = sciGetCurrentObj();
+  
   if (mark <= 0)
     { 
-      sciSetIsMark(pobj, TRUE);
-      sciSetIsLine(pobj,FALSE);
-      sciSetMarkStyle (pobj,abs(mark));
-/*       sciSetForeground (pobj, sciGetForeground (psubwin)); */
+      int absmark = abs(mark);
+      sciSetCurrentObj (ConstructPolyline(psubwin,x,y,PD0,closed,n,1,1,
+					  NULL,NULL,&absmark,NULL,NULL,FALSE,FALSE,TRUE,FALSE));
     }
   else
     {
-      sciSetIsMark(pobj, FALSE);
-      sciSetIsLine(pobj, TRUE);
-/*       sciSetLineStyle(pobj, sciGetLineStyle (psubwin)); */
-      sciSetForeground (pobj, mark);
+      sciSetCurrentObj (ConstructPolyline(psubwin,x,y,PD0,closed,n,1,1,
+					  &mark,NULL,NULL,NULL,NULL,TRUE,FALSE,FALSE,FALSE));
     }
-  *hdl=sciGetHandle(pobj); 
-  if (pSUBWIN_FEATURE(psubwin)->surfcounter>0){
-    Merge3d(psubwin); /* an addtomerge function should be much more efficient */
-    /*    EraseAndOrRedraw(sciGetSelectedSubWin (sciGetCurrentFigure ()));} */ /* inhibit EraseAndOrRedraw for now F.Leray 20.12.04 */
-    sciDrawObj(sciGetCurrentFigure ());}
-  else
-    sciDrawObjIfRequired(pobj);
   
+   pobj = sciGetCurrentObj();
+   *hdl=sciGetHandle(pobj); 
 }
   
 
@@ -115,76 +119,46 @@ void Objpoly (x,y,n,closed,mark,hdl)
  * Objfpoly : 
  *-----------------------------------------------*/
 
-void Objfpoly (x,y,n,style,hdl)
-     integer n,style;
+void Objfpoly (x,y,n,style,hdl,shading)
+     integer n, *style, shading;
      double *x,*y;
      long * hdl;
 { 
-  long hdltab[2];
   int fillcolor, contourcolor;
   sciPointObj *psubwin, *pobj;
   int closed = 1; /* we close the polyline by default */
   psubwin = (sciPointObj *)sciGetSelectedSubWin (sciGetCurrentFigure ());
   
-  
-  if(style == 0) /* filled color is the one used for axes background */
-    fillcolor = sciGetBackground(psubwin);
-  else /* fill with abs(style) */
-    fillcolor = abs(style);
-  
-  /*   if(style == 0) */
-  /*     closed = 0; */
-  /*   else */
-  /*     closed = 1; */
-  
-  sciSetCurrentObj (ConstructPolyline(psubwin,x,y,PD0,closed,n,1,5)); /* polyline_style is "filled" == 5 */
-  pobj = sciGetCurrentObj();
-  
-/*   if (style < 0) */
-/*     { */
-/*       sciSetIsMark(pobj, FALSE); */
-/*       sciSetIsLine(pobj, TRUE); */
-/*       sciSetForeground (pobj, abs(style)); */
-/*       sciSetBackground (pobj, abs(style)); */
-/*     } */
-/*   else if (style > 0) */
-/*     { */
-/*       sciSetIsMark(pobj, FALSE); */
-/*       sciSetIsLine(pobj, TRUE); */
-/*       sciSetBackground (pobj, style); */
-/*     } */
-/*   else */
-/*     { */
-/*       sciSetIsMark(pobj, FALSE); */
-/*       sciSetIsLine(pobj, TRUE); */
-/*       sciSetForeground (pobj, style); */
-/*     } */
-  
-  sciSetForeground (pobj, fillcolor);
 
-  hdltab[0]=sciGetHandle(pobj);
- 
-  if (style < 0)
-    contourcolor = fillcolor;
+  if(shading == 2)
+    {
+      /* interpolated shading is "on" */
+      sciSetCurrentObj (ConstructPolyline(psubwin,x,y,PD0,closed,n,
+					  1,1,NULL,style,NULL,NULL,NULL,FALSE,TRUE,FALSE,TRUE));
+    }
   else
-    contourcolor = sciGetForeground(psubwin);
-  
-  sciSetCurrentObj (ConstructPolyline(psubwin,x,y,PD0,closed,n,1,1));  /* polyline_style is "interpolated" == 0 */
-  pobj = sciGetCurrentObj();
-  sciSetForeground (pobj, contourcolor);
-
-  hdltab[1]=sciGetHandle(pobj);
-  sciSetCurrentObj(ConstructAgregation (hdltab, 2)); 
-/* } */
-  
-  if (pSUBWIN_FEATURE(psubwin)->surfcounter>0) {
-    Merge3d(psubwin); /* an addtomerge function should be much more efficient */
-    /*    EraseAndOrRedraw(sciGetSelectedSubWin (sciGetCurrentFigure ()));} /\* inhibit EraseAndOrRedraw for now F.Leray 20.12.04 *\/ */
-    sciDrawObj(sciGetCurrentFigure ());}
-  else
-    sciDrawObjIfRequired(sciGetCurrentObj ());
-  *hdl=sciGetHandle(sciGetCurrentObj ()); 
-  
+    {
+      /* flat mode is "on" */
+      if (*style < 0){
+	fillcolor = abs(*style);
+	sciSetCurrentObj (ConstructPolyline(psubwin,x,y,PD0,closed,n,
+					    1,1,NULL,&fillcolor,NULL,NULL,NULL,FALSE,TRUE,FALSE,FALSE));
+      }
+      else if (*style == 0){
+	contourcolor = sciGetForeground(psubwin);
+	sciSetCurrentObj (ConstructPolyline(psubwin,x,y,PD0,closed,n,
+					    1,1,&contourcolor,NULL,NULL,NULL,NULL,TRUE,FALSE,FALSE,FALSE));
+      }
+      else{ /* *style > 0*/
+	fillcolor = *style;
+	contourcolor = sciGetForeground(psubwin);
+	sciSetCurrentObj (ConstructPolyline(psubwin,x,y,PD0,closed,n,
+					    1,1,&contourcolor,&fillcolor,NULL,NULL,NULL,TRUE,TRUE,FALSE,FALSE));
+      }
+      
+      pobj = sciGetCurrentObj();
+      *hdl=sciGetHandle(sciGetCurrentObj ());  
+    }
 }
 
 
@@ -198,56 +172,50 @@ void Objsegs (style,flag,n1,x,y,arsize)
 { 
   integer type=0,n2, colored=0;
   double *fx,*fy,arfact=1.0;
-  sciPointObj *psubwin;
   int typeofchamp = -1; /* no champ here, only segs ; this info is useless */
-  psubwin = (sciPointObj *)sciGetSelectedSubWin (sciGetCurrentFigure ());
+  sciPointObj *psubwin = (sciPointObj *)sciGetSelectedSubWin (sciGetCurrentFigure ());
 
   n2=n1;
   fx=x;fy=y;
   sciSetCurrentObj (ConstructSegs(psubwin,type,
 				  x,y,n1,n2,fx,fy,flag,style,arsize,colored,arfact,typeofchamp)); 
-  if (pSUBWIN_FEATURE(psubwin)->surfcounter>0) {
-    Merge3d(psubwin); /* an addtomerge function should be much more efficient */
-    /*     EraseAndOrRedraw(sciGetSelectedSubWin (sciGetCurrentFigure ()));} /\* inhibit EraseAndOrRedraw for now F.Leray 20.12.04 *\/ */
-    sciDrawObj(sciGetCurrentFigure ());}
-  else
-    sciDrawObjIfRequired(sciGetCurrentObj ());  
-  
+  sciDrawObjIfRequired(sciGetCurrentObj ()); 
 }
 /*-----------------------------------------------------------
  * Objstring:
  *-----------------------------------------------------------*/
 
-void Objstring(fname,fname_len,str,x,y,angle,box,wh,fill,hdl)
-     char *fname; 
+/* box is an OUTPUT re-used inside matdes.c in scixstring */
+void Objstring(fname,fname_len,str,x,y,angle,box,wh,hdl,fill,foreground,background,isboxed,isline,isfilled)
+     char *fname;
      unsigned long fname_len; 
      integer str;
      double x,y,*angle,*box,*wh;
      int fill;
      long *hdl;
-{ 
-  integer v;
-  double dv;
+     int *foreground, *background;
+     BOOL isboxed,isline,isfilled;
+{
+  integer v ;
+  double dv ;
   integer x1,yy1,n=1,rect1[4];
   sciPointObj *psubwin, *pobj;
    
   psubwin = (sciPointObj *)sciGetSelectedSubWin (sciGetCurrentFigure ());
   sciSetCurrentObj (ConstructText
 			(psubwin, fname,
-			 strlen (fname), x, y,wh));
+			 strlen (fname), x, y,wh, fill,foreground,background,isboxed,isline,isfilled));
   pobj=sciGetCurrentObj ();
   *hdl= sciGetHandle(pobj);
-  sciSetFontOrientation (pobj, (int) (*angle *  10)); 
-  pTEXT_FEATURE (pobj)->fill=fill;
-/*   sciSetForeground (pobj, sciGetForeground (psubwin)); */
-/*   sciSetFontStyle(pobj, sciGetFontStyle (psubwin)); */
-/*   sciSetFontDeciWidth(pobj, sciGetFontDeciWidth (psubwin)); */
+  sciSetFontOrientation (pobj, (int) (*angle *  10));
+
   sciDrawObjIfRequired(pobj);
           
+  updateScaleIfRequired( sciGetParentSubwin( pobj ) ) ;
 
   x1 = XDouble2Pixel(x);
   yy1 = YDouble2Pixel(y);
-  C2F(dr)("xstringl",fname,&x1,&yy1,rect1,&v,&v,&v,&dv,&dv,&dv,&dv,9L,fname_len);
+  C2F(dr)("xstringl",fname,&x1,&yy1,rect1,&v,&v,&v,&dv,&dv,&dv,&dv,9L,fname_len) ;
   C2F(echelle2d)(box,box+1,rect1,rect1+1,&n,&n,"i2f",3L);
   C2F(echelle2dl)(box+2,box+3,rect1+2,rect1+3,&n,&n,"i2f");
  
@@ -349,8 +317,13 @@ void Objplot3d (fname,isfac,izcol,x,y,z,zcol,m,n,theta,alpha,legend,iflag,ebox,m
   char * legx = NULL;
   char * legy = NULL;
   char * legz = NULL;
-/*   char * buff = NULL; */
+  /*   char * buff = NULL; */
+  int flag_x = 1;
+  int flag_y = 1;
+  int dimvectx = -1;
+  int dimvecty = -1;
 
+  sciPointObj  * pNewSurface = NULL ;
   sciSubWindow * ppsubwin = NULL;
   BOOL bounds_changed = FALSE; /* cannot be used here because we have to force redrawing since there is no way to avoid merge (=> complete redraaw) */
 
@@ -473,10 +446,10 @@ void Objplot3d (fname,isfac,izcol,x,y,z,zcol,m,n,theta,alpha,legend,iflag,ebox,m
   pSUBWIN_FEATURE (psubwin)->theta  = *theta; 
 
   ok=0;  
- if (pSUBWIN_FEATURE(psubwin)->surfcounter>0) ok=1;
-
+  if (ChildrenCounter(psubwin)>0) ok=1;
+  
   if ((sciGetGraphicMode (psubwin)->autoscaling)) {
-     /* compute and merge new specified bounds with psubwin->Srect */
+    /* compute and merge new specified bounds with psubwin->Srect */
     switch (iflag[1])  {
     case 0:  /* do not change psubwin->Srect */
       break;
@@ -500,16 +473,16 @@ void Objplot3d (fname,isfac,izcol,x,y,z,zcol,m,n,theta,alpha,legend,iflag,ebox,m
       drect[5] = (double) Maxi(z, mn); /*zmax*/
       break;
     }
-   if (!pSUBWIN_FEATURE(psubwin)->FirstPlot ) { /* merge psubwin->Srect and drect */
+    if (!pSUBWIN_FEATURE(psubwin)->FirstPlot ) { /* merge psubwin->Srect and drect */
       drect[0] = Min(pSUBWIN_FEATURE(psubwin)->SRect[0],drect[0]); /*xmin*/
       drect[1] = Max(pSUBWIN_FEATURE(psubwin)->SRect[1],drect[1]); /*xmax*/
       drect[2] = Min(pSUBWIN_FEATURE(psubwin)->SRect[2],drect[2]); /*ymin*/
       drect[3] = Max(pSUBWIN_FEATURE(psubwin)->SRect[3],drect[3]); /*ymax*/
       drect[4] = Min(pSUBWIN_FEATURE(psubwin)->SRect[4],drect[4]); /*zmin*/
       drect[5] = Max(pSUBWIN_FEATURE(psubwin)->SRect[5],drect[5]); /*zmax*/
-   }
-   if (iflag[1] != 0) 
-     bounds_changed = update_specification_bounds(psubwin, drect,3);
+    }
+    if (iflag[1] != 0) 
+      bounds_changed = update_specification_bounds(psubwin, drect,3);
   } 
   
   if(iflag[1] != 0)
@@ -522,14 +495,14 @@ void Objplot3d (fname,isfac,izcol,x,y,z,zcol,m,n,theta,alpha,legend,iflag,ebox,m
 
   if (*isfac== 1) { 
     if (*izcol == 0) { 
-      if (strncmp(fname,"plot3d1",7)==0) {typeof3d = SCI_FAC3D;   flagcolor=1;}
+      if (strcmp(fname,"plot3d1")==0) {typeof3d = SCI_FAC3D;   flagcolor=1;}
       else                               {typeof3d = SCI_FAC3D;   flagcolor=0;}
     }
     else if (*izcol == 2)                {typeof3d = SCI_FAC3D;   flagcolor=3;}
     else                                 {typeof3d = SCI_FAC3D;   flagcolor=2;}
   } 
   else  if  (*isfac== 0) {
-    if (strncmp(fname,"plot3d1",7)==0)   {typeof3d = SCI_PLOT3D;  flagcolor=1;}
+    if (strcmp(fname,"plot3d1")==0)   {typeof3d = SCI_PLOT3D;  flagcolor=1;}
     else                                 {typeof3d = SCI_PLOT3D;  flagcolor=0;}
   }
   else                                   {typeof3d = SCI_PARAM3D1;flagcolor=1;}
@@ -539,45 +512,181 @@ void Objplot3d (fname,isfac,izcol,x,y,z,zcol,m,n,theta,alpha,legend,iflag,ebox,m
    * Construct the Entities
    * ================================================= */
 
-  if ( typeof3d != SCI_PARAM3D1 ) /*Distinction here between SCI_PARAM3D1 and others*/
-    sciSetCurrentObj (ConstructSurface
-		      ((sciPointObj *)
-		       psubwin, typeof3d,
-		       x, y, z, zcol, *izcol, *m, *n, iflag,ebox,flagcolor,
-		       isfac,m1,n1,m2,n2,m3,n3,m3n,n3n));
+  if ( typeof3d != SCI_PARAM3D1 ) {/*Distinction here between SCI_PARAM3D1 and others*/
+
+    if ( *isfac == 1 )
+    {
+      /* x is considered as a matrix */
+      dimvectx = -1 ;
+    }
+    else if(*m1 == 1)
+    { /* x is a row vector */
+      dimvectx = *n1;
+    }
+    else if(*n1 == 1)
+    { /* x is a column vector */
+      dimvectx = *m1;
+    }
+    else
+    { 
+      dimvectx = -1;
+    }
+    
+    if(dimvectx>1){
+      /* test the monotony on x*/
+      if(x[0] >= x[1]) /* decreasing */
+	{
+	  int i;
+	  for(i=1;i<dimvectx-1;i++)
+	    {
+	      if(x[i] < x[i+1])
+		{
+		  Scierror(999,"Objplot3d: x vector is not monotonous \t\n");
+		  return;
+		}
+	    }
+	  flag_x = -1;
+	}
+      else /* x[0] < x[1]*/
+	{
+	  for(i=1;i<dimvectx-1;i++)
+	    {
+	      if(x[i] > x[i+1])
+		{
+		  Scierror(999,"Objplot3d: x vector is not monotonous \t\n");
+		  return;
+		}
+	    }
+	  flag_x = 1;
+	}
+    }
+    
+    if ( *isfac == 1 )
+    {
+      /* y is considered as a matrix */
+      dimvecty = -1 ;
+    }
+    else if(*m2 == 1) /* y is a row vector */
+    {
+      dimvecty = *n2;
+    }
+    else if(*n2 == 1) /* y is a column vector */
+    {
+      dimvecty = *m2;
+    }
+    else /* y is a matrix */
+    {
+      dimvecty = -1;
+    }
+   
+    if(dimvecty>1){
+      /* test the monotony on y*/
+      if(y[0] >= y[1]) /* decreasing */
+	{
+	  int i;
+	  for(i=1;i<dimvecty-1;i++)
+	  {
+	    if(y[i] < y[i+1])
+	      {
+		Scierror(999,"Objplot3d: y vector is not monotonous \t\n");
+		return;
+	      }
+	  }
+	  flag_y = -1;
+	}
+      else /* y[0] < y[1]*/
+	{
+	  for(i=1;i<dimvecty-1;i++)
+	    {
+	      if(y[i] > y[i+1])
+		{
+		  Scierror(999,"Objplot3d: y vector is not monotonous \t\n");
+		  return;
+		}
+	    }
+	  flag_y = 1;
+	}
+    }
+
+    pNewSurface = ConstructSurface( psubwin, typeof3d,
+                                    x, y, z, zcol, *izcol, *m, *n, iflag,ebox,flagcolor,
+                                    isfac,m1,n1,m2,n2,m3,n3,m3n,n3n) ;
+    
+    if ( pNewSurface == NULL ) { return ; }
+
+    sciSetCurrentObj( pNewSurface ) ;
+        
+    pSURFACE_FEATURE (sciGetCurrentObj())->flag_x = flag_x;
+    pSURFACE_FEATURE (sciGetCurrentObj())->flag_y = flag_y;
+    
+    
+  }
   else
     {
-      if ((hdltab = malloc (*n * sizeof (long))) == NULL) {
+      if ((hdltab = MALLOC (*n * sizeof (long))) == NULL) {
 	Scierror(999,"%s: No more memory available\r\n",fname);
 	return; 
       }
       for (i = 0; i < *n; ++i) { 
 	/* F.Leray Pb here: In fact we do not create a Surface but one or several 3D Polylines
-	 Pb comes when wanting to access the fields "surface_color" or "flag" for example
-	 in function sciSet (cf. matdes.c). 
-	 Question 1: Are these properties accessible from a SCI_PARAM3D1 ?
-	 Question 2: Is "flag" obsolete and replaced by "color_mode"?*/
-	sciSetCurrentObj (ConstructPolyline
-			  ((sciPointObj *)sciGetSelectedSubWin (sciGetCurrentFigure ()),
-			   &(x[*m * i]),&(y[*m * i]),&(z[*m * i]),0,*m,1,1));  
-	pobj = sciGetCurrentObj();
+	   Pb comes when wanting to access the fields "surface_color" or "flag" for example
+	   in function sciSet (cf. matdes.c). 
+	   Question 1: Are these properties accessible from a SCI_PARAM3D1 ?
+	   Question 2: Is "flag" obsolete and replaced by "color_mode"?*/
+
+	/* 	sciSetCurrentObj (ConstructPolyline */
+	/* 			  ((sciPointObj *)sciGetSelectedSubWin (sciGetCurrentFigure ()), */
+	/* 			   &(x[*m * i]),&(y[*m * i]),&(z[*m * i]),0,*m,1,1));   */
+
 	if ((*n > 0) && (zcol != (double *)NULL)) {
 	  if ((int) zcol[i] > 0){
-	    sciSetForeground (pobj, (int) zcol[i]);
-	    sciSetIsMark(pobj, FALSE);
-	    sciSetIsLine(pobj,  TRUE);
+	    int intzcol = (int) zcol[i];
+	    sciSetCurrentObj (ConstructPolyline
+			      ((sciPointObj *)sciGetSelectedSubWin (sciGetCurrentFigure ()),
+			       &(x[*m * i]),&(y[*m * i]),&(z[*m * i]),0,*m,*n,1,
+			       &intzcol,NULL,NULL,NULL,NULL,TRUE,FALSE,FALSE,FALSE));  
 	  }
 	  else {
-	    sciSetMarkSizeUnit(pobj,2); /* force switch to tabulated mode : old syntax */
- 	    sciSetIsMark(pobj,TRUE);
-	    sciSetIsLine(pobj,FALSE);
-	    sciSetMarkStyle(pobj,(int) -zcol[i]);
+	    int intzcol = (int) -zcol[i];
+	    sciSetCurrentObj (ConstructPolyline
+			      ((sciPointObj *)sciGetSelectedSubWin (sciGetCurrentFigure ()),
+			       &(x[*m * i]),&(y[*m * i]),&(z[*m * i]),0,*m,*n,1,
+			       NULL,NULL,&intzcol,NULL,NULL,FALSE,FALSE,TRUE,FALSE));  
 	  }
 	}
+	else { /* default case, nothing is given */
+	  int curcolor = sciGetForeground(sciGetSelectedSubWin(sciGetCurrentFigure ())); /* current color equivalent for new graphics mode */
+	  sciSetCurrentObj (ConstructPolyline
+			    ((sciPointObj *)sciGetSelectedSubWin (sciGetCurrentFigure ()),
+			     &(x[*m * i]),&(y[*m * i]),&(z[*m * i]),0,*m,*n,1,
+			     &curcolor,NULL,NULL,NULL,NULL,TRUE,FALSE,FALSE,FALSE));  
+	}
+
+	pobj = sciGetCurrentObj();
+	
 	hdltab[i]=sciGetHandle(pobj); 
-      } 
-      /** construct agregation and make it current object**/
-      if ( *n>1 ) sciSetCurrentObj (ConstructAgregation (hdltab, *n));  
+      }
+
+
+/*       	if ((*n > 0) && (zcol != (double *)NULL)) { */
+/*       	  if ((int) zcol[i] > 0){ */
+/*       	    sciSetForeground (pobj, (int) zcol[i]); */
+/*       	    sciSetIsMark(pobj, FALSE); */
+/*       	    sciSetIsLine(pobj,  TRUE); */
+/*       	  } */
+/*       	  else { */
+/*       /\* 	    sciSetMarkSizeUnit(pobj,2); /\\* force switch to tabulated mode : old syntax *\\/ *\/ */
+/*        	    sciSetIsMark(pobj,TRUE); */
+/*       	    sciSetIsLine(pobj,FALSE); */
+/*       	    sciSetMarkStyle(pobj,(int) -zcol[i]); */
+/*       	  } */
+/*       	} */
+/*       	hdltab[i]=sciGetHandle(pobj); */
+/*             } */
+
+
+      /** construct Compound and make it current object**/
+      if ( *n>1 ) sciSetCurrentObj (ConstructCompound (hdltab, *n));  
       FREE(hdltab);
     }
 
@@ -590,9 +699,11 @@ void Objplot3d (fname,isfac,izcol,x,y,z,zcol,m,n,theta,alpha,legend,iflag,ebox,m
   /* =================================================
    * Redraw Figure
    * ================================================= */
-
-  sciDrawObj(sciGetCurrentFigure ());
-/*   EraseAndOrRedraw(sciGetSelectedSubWin (sciGetCurrentFigure ())); /\* inhibit EraseAndOrRedraw for now F.Leray 20.12.04 *\/ */
+  {
+    sciPointObj * pfig = sciGetCurrentFigure () ;
+    sciDrawObj( pfig ) ;
+  }
+  /*   EraseAndOrRedraw(sciGetSelectedSubWin (sciGetCurrentFigure ())); /\* inhibit EraseAndOrRedraw for now F.Leray 20.12.04 *\/ */
   pSUBWIN_FEATURE(psubwin)->FirstPlot=FALSE;
    
   FREE(loc); loc = NULL;
