@@ -18,35 +18,37 @@ c
       iadr(l)=l+l-1
       sadr(l)=(l/2)+1
 c     
-c     nordre est le numero d'ordre de cet external dans la structure
-c     de donnee,
-c     mlhs (mrhs) est le nombre de parametres de sortie (entree)
-c     du simulateur 
-c     
+c     nordre=external number
+c     mlhs (mrhs) = number ot output (input) parameters of the 
+c     external 
       iero=0
       mrhs=3
 c     
       ilp=iadr(lstk(top))
       il=istk(ilp+nordre)
+
+c     external is a Scilab function
+c     on return iero=1 is used to notify to the ode solver that
+c     scilab was not able to evaluate the external
+      iero=1
+
 c     
-c     transfert des arguments d'entree minimaux du simulateur
-c     la valeur de ces arguments vient du contexte fortran (liste d'appel)
-c     la structure vient du contexte 
-c+    
+c     transfer of input parameters
+c+     
       call ftob(t,1,istk(il+1))
-      if(err.gt.0) goto 9999
+      if(err.gt.0.or.err1.gt.0) return
       call ftob(y,ny,istk(il+2))
-      if(err.gt.0) goto 9999
+      if(err.gt.0.or.err1.gt.0) return
 c     
       top=top+1
       if(top+1.ge.bot) then
          call error(18)
-         if(err.gt.0) return
+         return
       endif
       err=lstk(top)+2+ny*ny-lstk(bot)
       if(err.gt.0) then
          call error(17)
-         if(err.gt.0) return
+         return
       endif
       ilp1=iadr(lstk(top))
       istk(ilp1)=1
@@ -62,22 +64,20 @@ c
       tops=istk(il)
       ils=iadr(lstk(tops))
       if(istk(ils).eq.15) goto 10
-c     
-c     recuperation de l'adresse du simulateur
+c+    
+c     adress of external
       fin=lstk(tops)
 c     
       goto 40
-c     cas ou le simulateur est decrit par une liste
+c     
+c     external in a list
  10   nelt=istk(ils+1)
       l=sadr(ils+3+nelt)
       ils=ils+2
-c     
-c     recuperation de l'adresse du simulateur
+c     adress of external
       fin=l
 c     
-c     gestion des parametres supplementaires du simulateur
-c     proviennent du contexte  (elements de la liste
-c     decrivant le simulateur
+c     additional parameters
 c     
       nelt=nelt-1
       if(nelt.eq.0) goto 40
@@ -85,12 +85,12 @@ c
       vol=istk(ils+nelt+1)-istk(ils+1)
       if(top+1+nelt.ge.bot) then
          call error(18)
-         if(err.gt.0) goto 9999
+         return
       endif
       err=lstk(top+1)+vol-lstk(bot)
       if(err.gt.0) then
          call error(17)
-         if(err.gt.0) goto 9999
+         return
       endif
       call unsfdcopy(vol,stk(l),1,stk(lstk(top+1)),1)
       do 11 i=1,nelt
@@ -100,13 +100,12 @@ c
       mrhs=mrhs+nelt
  40   continue
 c     
-c     execution de la macro definissant le simulateur
+c     execute scilab external
 c     
-      iero=0
       pt=pt+1
       if(pt.gt.psiz) then
          call error(26)
-         goto 9999
+         return
       endif
       ids(1,pt)=lhs
       ids(2,pt)=rhs
@@ -123,24 +122,24 @@ c
  200  lhs=ids(1,pt)
       rhs=ids(2,pt)
       pt=pt-1
-c+    
-c     transfert des variables  de sortie vers fortran
-      call btof(p,ny*ny)
-      if(err.gt.0) goto 9999
-c+    
       niv=niv-1
+c+    
+c     transfer of output parameters of external to fortran
+      call btof(p,ny*ny)
+      if(err.gt.0.or.err1.gt.0) return
+c+    
+c     normal return
+      iero=0
       return
 c     
  9999 continue
+      niv=niv-1
       if(err1.gt.0) then
+c     .  the error has been catched
          lhs=ids(1,pt)
          rhs=ids(2,pt)
          pt=pt-1
          fun=0
       endif
-      iero=1
-      niv=niv-1
       return
-
       end
-
