@@ -31,12 +31,17 @@ c
       ilp=iadr(lstk(top))
       il=istk(ilp+nordre)
 c     
-c     transfert des arguments d'entree minimaux du simulateur
-c     la valeur de ces arguments vient du contexte fortran (liste d'appel)
-c     la structure vient du contexte.
-c+    
-      call ftob(xyz,3,istk(il+1))
+c     external is a Scilab function
 
+c     on return iero=1 is used to notify to the ode solver that
+c     scilab was not able to evaluate the external
+      iero=1
+
+c     Putting Fortran arguments on Scilab stack 
+
+
+      call ftob(xyz,3,istk(il+1))
+      if(err.gt.0.or.err1.gt.0) return
 c      call ftob(numfun,1,istk(il+2))
       hsize=4
       if(top.ge.bot) then
@@ -86,12 +91,12 @@ c
       vol=istk(ils+nelt+1)-istk(ils+1)
       if(top+1+nelt.ge.bot) then
          call error(18)
-         if(err.gt.0) goto 9999
+         return
       endif
       err=lstk(top+1)+vol-lstk(bot)
       if(err.gt.0) then
          call error(17)
-         if(err.gt.0) goto 9999
+         return
       endif
       call unsfdcopy(vol,stk(l),1,stk(lstk(top+1)),1)
       do 11 i=1,nelt
@@ -103,7 +108,6 @@ c
 c     
 c     execution de la macro definissant le simulateur
 c     
-      iero=0
       pt=pt+1
       if(pt.gt.psiz) then
          call error(26)
@@ -125,22 +129,25 @@ c
  200  lhs=ids(1,pt)
       rhs=ids(2,pt)
       pt=pt-1
+      niv=niv-1
+      
 c+    
 c     transfert des variables  de sortie vers fortran
       call btof(v,numfun)
-      if(err.gt.0) goto 9999
-c+    
-      niv=niv-1
+      if(err.gt.0.or.err1.gt.0) return
+c     normal return iero set to 0
+      iero=0 
+      return
+
       return
 c     
  9999 continue
+      niv=niv-1
       if(err1.gt.0) then
          lhs=ids(1,pt)
          rhs=ids(2,pt)
          pt=pt-1
          fun=0
       endif
-      iero=1
-      niv=niv-1
       return
       end
