@@ -7,16 +7,17 @@
 int C2F(intTclUpVar) _PARAMS((char *fname))
 {
 	static int l1,n1,m1;
-	int TypeVar1=GetType(1);
-	int TypeVar2=GetType(2);
+	static int l2,n2,m2;
+	
+	Tcl_Interp *TCLinterpreter=NULL;
 
-	CheckRhs(2,2);
+	CheckRhs(2,3);
 	CheckLhs(0,1);
 
-	if ( (TypeVar1 == sci_strings) && (TypeVar2 == sci_strings) )
+	if ( (GetType(1) == sci_strings) && (GetType(2) == sci_strings) )
 	{
 		char *sourceName=NULL,*destName=NULL;
-		int *paramoutINT=(int*)malloc(sizeof(int));
+		int *paramoutINT=(int*)MALLOC(sizeof(int));
 		
 		GetRhsVar(1,"c",&m1,&n1,&l1);
 		sourceName=cstk(l1);
@@ -24,7 +25,38 @@ int C2F(intTclUpVar) _PARAMS((char *fname))
 		GetRhsVar(2,"c",&m1,&n1,&l1);
 		destName=cstk(l1);
 
-		if ( Tcl_UpVar(TCLinterp,"#0", sourceName, destName, TCL_GLOBAL_ONLY) == TCL_ERROR )
+		if (TCLinterp == NULL)
+		{
+			Scierror(999,TCL_ERROR13,fname);
+			return 0;
+		}
+
+		if (Rhs==3)
+		{
+			/* three arguments given - get a pointer on the slave interpreter */
+			if (GetType(3) == sci_strings)
+			{
+				GetRhsVar(3,"c",&m2,&n2,&l2)
+				TCLinterpreter=Tcl_GetSlave(TCLinterp,cstk(l2));
+				if (TCLinterpreter==NULL)
+				{
+					Scierror(999,TCL_ERROR17,fname);
+					return 0;
+				}
+			}
+			else
+			{
+				 Scierror(999,TCL_ERROR14,fname);
+				 return 0;
+			}
+		}
+		else
+		{
+			/* only two arguments given - use the main interpreter */
+			TCLinterpreter=TCLinterp;
+		}
+
+		if ( Tcl_UpVar(TCLinterpreter,"#0", sourceName, destName, TCL_GLOBAL_ONLY) == TCL_ERROR )
 		{
 			*paramoutINT=(int)(FALSE);
 		}
@@ -36,12 +68,12 @@ int C2F(intTclUpVar) _PARAMS((char *fname))
 		CreateVarFromPtr(1, "b", &n1, &n1, &paramoutINT);
 		LhsVar(1)=1;
 		C2F(putlhsvar)();
-		if (paramoutINT) {free(paramoutINT);paramoutINT=NULL;}
+		if (paramoutINT) {FREE(paramoutINT);paramoutINT=NULL;}
 
 	}
 	else
 	{
-		Scierror(999,"TCL_UpVar : Invalid Argument type.\r\n");
+		Scierror(999,TCL_ERROR14,fname);
 		return 0;
 	}
 	
