@@ -1,5 +1,32 @@
 #include "wgmenu.h"
+#include <direct.h>
 
+#include "Messages.h"
+#include "Warnings.h"
+#include "Errors.h"
+
+#include "../os_specific/win_mem_alloc.h" /* MALLOC */
+/*-----------------------------------------------------------------------------------*/
+extern HINSTANCE hdllInstance;
+/*-----------------------------------------------------------------------------------*/
+extern BOOL IsWindowInterface();
+extern void Write_Scilab_Console (char *buf);
+extern void Write_Scilab_Window (char *buf);
+extern void ResetMenu(void);
+extern struct BCG *GetWindowXgcNumber (integer i);
+extern void HideGraphToolBar(struct BCG * ScilabGC);
+extern void ShowGraphToolBar(struct BCG * ScilabGC);
+extern void SaveCurrentLine(BOOL RewriteLineAtPrompt);
+extern void ExportBMP(struct BCG *ScilabGC,char *pszflname);
+extern void ExportEMF(struct BCG *ScilabGC,char *pszflname);
+extern HDC GetPrinterDC(void);
+extern char GetPrinterOrientation(void);
+extern void PrintPs (struct BCG *ScilabGC);
+extern void SavePs (struct BCG *ScilabGC);
+extern char *GetScilabDirectory(BOOL UnixStyle);
+extern void Callback_PRINTSETUP(void);
+extern LPTW GetTextWinScilab(void);
+void ModifyEntityPickerToolbar(struct BCG * ScilabGC,BOOL Pressed);
 /*-----------------------------------------------------------------------------------*/
 /*********************************
  * Send a macro to the text window 
@@ -55,18 +82,22 @@ void SendGraphMacro (struct BCG *ScilabGC, UINT m)
 		  
 	      s++;
 	      break;
+		case PRINTSETUP:
+			Callback_PRINTSETUP();
+			s++;
+			break;
 	    case PRINT:
 	      CopyPrint (ScilabGC);
 	      s++;
 	      break;
 	    case COPYCLIP:
 	      {
-		NewCopyClip (ScilabGC);
+		CopyToClipboardEMF (ScilabGC);
 		s++;
 	      }
 	      break;
 	    case COPYCLIP1:
-	      CopyClip (ScilabGC);
+	      CopyToClipboardBitmap (ScilabGC);
 	      s++;
 	      break;
 	    case REDRAW:
@@ -78,8 +109,18 @@ void SendGraphMacro (struct BCG *ScilabGC, UINT m)
 	      s++;
 	      break;
 	    case SCIPS:
-	      SavePs (ScilabGC);
-	      s++;
+		{
+   			char *SaveCurrentPath=NULL;
+			SaveCurrentPath=_getcwd( NULL, 0 );
+			SavePs (ScilabGC);
+			if (SaveCurrentPath)
+			{
+				_chdir(SaveCurrentPath);
+				free(SaveCurrentPath); /* here must be a "little" free with _getcwd*/
+				SaveCurrentPath=NULL;
+			}
+			s++;
+		}
 	      break;
 	    case SCIPR:
 	      PrintPs (ScilabGC);
@@ -148,53 +189,77 @@ void SendGraphMacro (struct BCG *ScilabGC, UINT m)
  * special buf expression ( i.e begining with '@' )
  *    are parsed 
  ******************************/
-
 void ScilabMenuAction (char *buf)
 {
-  char *d;
-  if (strlen (buf) > 2 && buf[0] == '@')
-    {
-      /** special cases : buf was produced by a dynamic menu **/
-      if (buf[1] == '0')
+	if (strlen (buf) > 2 && buf[0] == '@')
 	{
-	  /* Interpreted mode : we store the action on a queue */
-	  StoreCommand1 (buf + 2, 0);
-	
-	}
-      else
-	{
-	  /* hard coded mode */
-	  int rep, win, entry;
-	  if ((d = strchr (buf, '_')) != NULL)
-	    {
-	      sscanf (d + 1, "%d(%d)", &win, &entry);
-	      entry--;
-	      *d = '\0';
-	    }
-	  else
-	    {
-	      win = -1;
-	      if ((d = strchr (buf, '(')) != NULL)
+	      /** special cases : LocalBuf was produced by a dynamic menu **/
+	    if (buf[1] == '0')
 		{
-		  sscanf (d, "(%d)", &entry);
-		  entry--;
-		  *d = '\0';
+		  /* Interpreted mode : we store the action on a queue */
+		  StoreCommand1 (buf + 2, 0);
+		}
+	    else
+		{
+			/* hard coded mode */
+			int rep, win, entry;
+			char *tmp=NULL;
+			char *LocalBuf=(char*)MALLOC((strlen(buf)+1)*sizeof(char));
+			char *FunctionName=(char*)MALLOC((strlen(buf)+1)*sizeof(char));
+			wsprintf(LocalBuf,"%s",buf+strlen("@1")+strlen("execstr("));
+			/*remove last char ')' */
+			LocalBuf[strlen(LocalBuf)-1]='\0';
+
+			if ( ( tmp = strchr (LocalBuf, '_') ) != NULL )
+			{
+				sscanf (tmp + 1, "%d(%d)", &win, &entry);
+				entry--;
+				strncpy(FunctionName,LocalBuf,strlen(LocalBuf)-strlen(tmp));
+				*tmp = '\0';
+			}
+			else
+			{
+				win = -1;
+				if ((tmp = strchr (LocalBuf, '(')) != NULL)
+				{
+					char *tmpbis=NULL;
+					sscanf (tmp, "(%d)", &entry);
+					
+					if ( ( tmpbis = strchr (LocalBuf, '(') ) != NULL )
+					{
+						strncpy(FunctionName,LocalBuf,strlen(LocalBuf)-strlen(tmpbis));
+					}
+
+					entry--;
+					*tmp = '\0';
+				}
+			}
+
+			C2F (setfbutn) (FunctionName, &rep);
+			if (rep == 0)
+			{
+				F2C (fbutn) (FunctionName, &(win), &(entry));
+			}
+
+			if (FunctionName) {FREE(FunctionName);FunctionName=NULL;}
+			if (LocalBuf) {FREE(LocalBuf);LocalBuf=NULL;}
+		}
+	}
+	else 
+	{
+		if (buf[0] != '\0')
+		{
+			/** standard string that we send to scilab **/
+			if (strlen (buf) == 1 && buf[0] <= 31 && buf[0] >= 1) 
+			{
+				StoreCommand1 (buf, 0);
+			}
+			else
+			{
+				StoreCommand1 (buf, 1);
+			}
 		}
-	    }
-	  /** removing leading @1execstr( **/
-	  C2F (setfbutn) (buf + 2 + 8, &rep);
-	  if (rep == 0)
-	    F2C (fbutn) (buf + 2, &(win), &(entry));
 	}
-    }
-  else if (buf[0] != '\0')
-    {
-      /** standard string that we send to scilab **/
-      if (strlen (buf) == 1 && buf[0] <= 31 && buf[0] >= 1)
-	StoreCommand1 (buf, 0);
-      else
-	StoreCommand1 (buf, 1);
-    }
 }
 /*-----------------------------------------------------------------------------------*/
 /************************************
@@ -208,10 +273,14 @@ void write_scilab (char *buf)
   
 }
 /*-----------------------------------------------------------------------------------*/
+void CallTranslateMacroGraphicWindow(char *string)
+{
+	TranslateMacro(string);
+}
+/*-----------------------------------------------------------------------------------*/
 /************************************
  * Translate string to tokenized macro 
  ************************************/
-
 static void TranslateMacro (char *string)
 {
   int i, len;
@@ -282,7 +351,7 @@ void LoadGraphMacros (struct BCG *ScilabGC)
 	  if (!(nInc = GetLine (buf, MAXSTR, menufile)))
 	    {
 	      nLine += nInc;
-	      BUGGOTOCLEAN ("Problem on line %d of %s\n");
+	      BUGGOTOCLEAN (MSG_WARNING5);
 	    }
 	  LeftJustify (buf, buf);
 	  if (nMenuLevel < MENUDEPTH)
@@ -291,7 +360,7 @@ void LoadGraphMacros (struct BCG *ScilabGC)
 	    }
 	  else
 	    {
-	      BUGGOTOCLEAN ("Menu is too deep at line %d of %s\n");
+	      BUGGOTOCLEAN (MSG_WARNING6);
 	    }
 	  hMenu[nMenuLevel] = CreateMenu ();
 	  AppendMenu (hMenu[nMenuLevel > 0 ? nMenuLevel - 1 : 0],
@@ -309,7 +378,7 @@ void LoadGraphMacros (struct BCG *ScilabGC)
 	  /* menu item */
 	  if (nCountMenu >= NUMMENU)
 	    {
-	      BUGGOTOCLEAN ("Too many menu items at line %d of %s\n");
+	      BUGGOTOCLEAN ( MSG_WARNING7 );
 	    }
 	  LeftJustify (buf, buf);
 	  if (buf[0] == '-')
@@ -330,7 +399,7 @@ void LoadGraphMacros (struct BCG *ScilabGC)
 	      if (!(nInc = GetLine (buf, MAXSTR, menufile)))
 		{
 		  nLine += nInc;
-		  BUGGOTOCLEAN ("Problem on line %d of %s\n");
+		  BUGGOTOCLEAN (MSG_WARNING8);
 		}
 	      LeftJustify (buf, buf);
 	      TranslateMacro (buf);
@@ -340,7 +409,7 @@ void LoadGraphMacros (struct BCG *ScilabGC)
 		}
 	      else
 		{
-		  BUGGOTOCLEAN ("Out of space for storing menu macros\n at line %d of %s\n");
+		  BUGGOTOCLEAN (MSG_WARNING9);
 		}
 	      ScilabGC->lpmw.macro[nCountMenu] = macroptr;
 	    }
@@ -357,7 +426,7 @@ void LoadGraphMacros (struct BCG *ScilabGC)
   goto cleanup;
 
  nomemory:
-  MessageBox (ScilabGC->hWndParent, "Out of memory",
+  MessageBox (ScilabGC->hWndParent, MSG_ERROR77,
 	      ScilabGC->lpgw->Title, MB_ICONEXCLAMATION);
 
  errorcleanup:
@@ -385,7 +454,7 @@ void LoadGraphMacros (struct BCG *ScilabGC)
 }
 /*-----------------------------------------------------------------------------------*/
 /************************************
- * Find a free position for storing 
+ * Find a FREE position for storing 
  * a new menu data
  ************************************/
 int WGFindMenuPos (BYTE ** macros)
@@ -678,103 +747,107 @@ void AddMenu (integer * win_num, char *button_name, char **entries,
   int number, nCountMenu;
   number = *win_num;
   if (*win_num == -1)
-    {
+  {
       lpmw = textwin.lpmw;
       hwnd = textwin.hWndParent;
       nCountMenu = WGFindMenuPos (lpmw->macro);
       if (nCountMenu >= NUMMENU)
-	{
-	  sciprint ("Can't add a menu item in scilab main menu \r\n");
-	  return;
-	}
-    }
+			{
+				sciprint (MSG_WARNING10);
+				return;
+			}
+  }
   else
-    {
-      ScilabGC = GetWindowXgcNumber (*win_num);
-      if (ScilabGC != (struct BCG *) 0)
-	{
-	  lpmw = &(ScilabGC->lpmw);
-	  hwnd = ScilabGC->hWndParent;
-	  nCountMenu = WGFindMenuPos (lpmw->macro);
-	  if (nCountMenu >= NUMMENU)
+  {
+     ScilabGC = GetWindowXgcNumber (*win_num);
+     if (ScilabGC != (struct BCG *) 0)
+		{
+			lpmw = &(ScilabGC->lpmw);
+			hwnd = ScilabGC->hWndParent;
+			nCountMenu = WGFindMenuPos (lpmw->macro);
+			if (nCountMenu >= NUMMENU)
 	    {
-	      sciprint ("Can't add a menu item in scilab graphic %d menu \r\n"
-			,*win_num);
+	      sciprint (MSG_WARNING11,*win_num);
 	      return;
 	    }
-	}
-      else
-	return;
-    }
+		}
+		else 	return;
+  }
+
   if (*ne == 0)
-    {
-      AppendMenu (lpmw->hMenu, MF_STRING, nCountMenu, button_name);
-      if ((macroptr = LocalAlloc (LPTR, lstrlen ("@%dexecstr(%s_%d(%d))")
-				  + lstrlen (fname) + 1)) != (BYTE *) NULL)
-	{
-	  if (*win_num < 0)
-	    if (*typ==0) 
-	      sprintf ((char *) macroptr, "@%dexecstr(%s(%d))", *typ, fname, 1);
-	    else if (*typ==2) 
-	      sprintf ((char *) macroptr, "@0%s(%d,%d)",fname, 1,*win_num);
-	    else
-	      sprintf ((char *) macroptr, "@%dexecstr(%s(%d))", *typ,fname, 1);
-	  else 
-	    if (*typ==0) 
-	      sprintf ((char *) macroptr, "@0execstr(%s_%d(%d))",fname, *win_num, 1);
-	    else if (*typ==2) 
-	      sprintf ((char *) macroptr, "@0%s(%d,%d)",fname, 1,*win_num);
-	    else
-	      sprintf ((char *) macroptr, "@%dexecstr(%s_%d(%d))", *typ,fname, *win_num, 1);
-	}
-      else
-	{
-	  sciprint ("Out of space for storing menu macros\n");
-	  return;
-	}
-      lpmw->macro[nCountMenu] = macroptr;
-    }
+  {
+    AppendMenu (lpmw->hMenu, MF_STRING, nCountMenu, button_name);
+    if ((macroptr = LocalAlloc (LPTR, lstrlen ("@%dexecstr(%s_%d(%d))")+ lstrlen (fname) + 1)) != (BYTE *) NULL)
+		{
+			if (*win_num < 0)
+				if (*typ==0) 
+					sprintf ((char *) macroptr, "@%dexecstr(%s(%d))", *typ, fname, 1);
+				else 
+					if (*typ==2) sprintf ((char *) macroptr, "@0%s(%d,%d)",fname, 1,*win_num);
+					else sprintf ((char *) macroptr, "@%dexecstr(%s(%d))", *typ,fname, 1);
+			else 
+				if (*typ==0) sprintf ((char *) macroptr, "@0execstr(%s_%d(%d))",fname, *win_num, 1);
+				else
+					if (*typ==2) sprintf ((char *) macroptr, "@0%s(%d,%d)",fname, 1,*win_num);
+					else sprintf ((char *) macroptr, "@%dexecstr(%s_%d(%d))", *typ,fname, *win_num, 1);
+		}
+    else
+		{
+			sciprint (MSG_WARNING12);
+			return;
+		}
+
+    lpmw->macro[nCountMenu] = macroptr;
+  }
   else
-    {
-      int i;
-      HMENU hMenu;
-      hMenu = CreateMenu ();
-      AppendMenu (lpmw->hMenu, MF_STRING | MF_POPUP, (UINT) hMenu, button_name);
-      for (i = 0; i < *ne; i++)
-	{
-	  nCountMenu = WGFindMenuPos (lpmw->macro);
-	  if (nCountMenu >= NUMMENU)
+  {
+		int i=0;
+		int j=0;
+    HMENU hMenu;
+    hMenu = CreateMenu ();
+		
+
+    AppendMenu (lpmw->hMenu, MF_STRING | MF_POPUP, (UINT) hMenu, button_name);
+    for (i = 0; i < *ne; i++)
+		{
+			
+			nCountMenu = WGFindMenuPos (lpmw->macro);
+			if (nCountMenu >= NUMMENU)
 	    {
-	      sciprint ("Can't add a menu item \r\n");
+	      sciprint (MSG_WARNING13);
 	      return;
 	    }
-	  AppendMenu (hMenu, MF_STRING, nCountMenu, entries[i]);
-	  if ((macroptr = LocalAlloc (LPTR, lstrlen ("@%dexecstr(%s_%d(%d))")
-				      + lstrlen (fname) + 1)) != (BYTE *) NULL)
-	    {
-	      if (*win_num < 0)
-		if (*typ==0) 
-		  sprintf ((char *) macroptr, "@%dexecstr(%s(%d))", *typ, fname, i + 1);
-		else if (*typ==2) 
-		  sprintf ((char *) macroptr, "@0%s(%d)",fname, i + 1);
-		else
-		  sprintf ((char *) macroptr, "@%dexecstr(%s(%d))", *typ,fname, i + 1);
-	      else
-		if (*typ==0) 
-		  sprintf ((char *) macroptr, "@0execstr(%s_%d(%d))",fname, *win_num, i + 1);
-		else if (*typ==2) 
-		  sprintf ((char *) macroptr, "@0%s(%d,%d)",fname, i + 1,*win_num);
-		else
-		  sprintf ((char *) macroptr, "@%dexecstr(%s_%d(%d))", *typ,fname, *win_num, i + 1);
-	    }
-	  else
-	    {
-	      sciprint ("Out of space for storing menu macros\n");
-	      return;
-	    }
-	  lpmw->macro[nCountMenu] = macroptr;
-	}
-    }
+			if ( strcmp("[--]",entries[i])==0 )
+			{
+				AppendMenu (hMenu, MF_SEPARATOR, 0, (LPSTR) NULL);
+			}
+			else
+			{
+				AppendMenu (hMenu, MF_STRING, nCountMenu, entries[i]);
+				if ((macroptr = LocalAlloc (LPTR, lstrlen ("@%dexecstr(%s_%d(%d))")+ lstrlen (fname) + 1)) != (BYTE *) NULL)
+				{
+					if (*win_num < 0)
+						if (*typ==0) 
+							sprintf ((char *) macroptr, "@%dexecstr(%s(%d))", *typ, fname, j + 1);
+						else
+							if (*typ==2) sprintf ((char *) macroptr, "@0%s(%d)",fname, j + 1);
+							else sprintf ((char *) macroptr, "@%dexecstr(%s(%d))", *typ,fname, j + 1);
+					else
+						if (*typ==0) sprintf ((char *) macroptr, "@0execstr(%s_%d(%d))",fname, *win_num, j + 1);
+						else
+							if (*typ==2) sprintf ((char *) macroptr, "@0%s(%d,%d)",fname, j + 1,*win_num);
+							else sprintf ((char *) macroptr, "@%dexecstr(%s_%d(%d))", *typ,fname, *win_num, j + 1);
+					j++;
+				}
+				else
+				{
+					sciprint (MSG_WARNING14);
+					return;
+				}
+				lpmw->macro[nCountMenu] = macroptr;
+			}
+		}
+  }
   DrawMenuBar (hwnd);
 }
 /*-----------------------------------------------------------------------------------*/
@@ -815,227 +888,7 @@ int C2F (addmen) (integer * win_num, char *button_name, integer * entries,
   return (0);
 }
 /*-----------------------------------------------------------------------------------*/
-/****************************************************
- * Event handler function for the line style window 
- * uses GetWindowLong(hwnd, 4) and SetWindowLong
- ****************************************************/
-
-EXPORT BOOL CALLBACK
-ExportStyleDlgProc (HWND hdlg, UINT wmsg, WPARAM wparam, LPARAM lparam)
-{
-  int i;
-  switch (wmsg)
-    {
-    case WM_INITDIALOG:
-      SendDlgItemMessage (hdlg, PS_COLOR, CB_ADDSTRING, 0,
-			  (LPARAM) ((LPSTR) "Black and white"));
-      SendDlgItemMessage (hdlg, PS_COLOR, CB_ADDSTRING, 0,
-			  (LPARAM) ((LPSTR) "Color"));
-      SendDlgItemMessage (hdlg, PS_COLOR, CB_SETCURSEL,
-			  ls.colored, 0L);
-      SendDlgItemMessage (hdlg, PS_LAND, CB_ADDSTRING, 0,
-			  (LPARAM) ((LPSTR) "Portrait"));
-      SendDlgItemMessage (hdlg, PS_LAND, CB_ADDSTRING, 0,
-			  (LPARAM) ((LPSTR) "Landscape"));
-      SendDlgItemMessage (hdlg, PS_LAND, CB_SETCURSEL,
-			  ls.land, 0L);
-      if (ls.use_printer == 0)
-	{
-	  for (i = 0; i < 8; i++)
-	    SendDlgItemMessage (hdlg, PS_TYPE, CB_ADDSTRING, 0,
-				(LPARAM) ((LPSTR) Print_Formats[i]));
-	  SendDlgItemMessage (hdlg, PS_TYPE, CB_SETCURSEL,
-			      ls.ps_type, 0L);
-	}
-      return TRUE;
-    case WM_COMMAND:
-      switch (LOWORD (wparam))
-	{
-	case PS_COLOR:
-	  ls.colored =
-	    (UINT) SendDlgItemMessage (hdlg, PS_COLOR, CB_GETCURSEL, 0, 0L);
-	  return FALSE;
-	case PS_TYPE:
-	  ls.ps_type =
-	    (UINT) SendDlgItemMessage (hdlg, PS_TYPE, CB_GETCURSEL, 0, 0L);
-	  return FALSE;
-	case PS_LAND:
-	  ls.land =
-	    (UINT) SendDlgItemMessage (hdlg, PS_LAND, CB_GETCURSEL, 0, 0L);
-	  return FALSE;
-	case IDOK:
-	  EndDialog (hdlg, IDOK);
-	  return TRUE;
-	case IDCANCEL:
-	  EndDialog (hdlg, IDCANCEL);
-	  return TRUE;
-	}
-      break;
-      return FALSE;
-    }
-  return FALSE;
-}
-/*-----------------------------------------------------------------------------------*/
-/****************************************************
- * Export style dialog box 
- ****************************************************/
-
-BOOL ExportStyle (struct BCG * ScilabGC)
-{
-  DLGPROC lpfnExportStyleDlgProc;
-  BOOL status = FALSE;
-  lpfnExportStyleDlgProc = (DLGPROC) MyGetProcAddress ("ExportStyleDlgProc",
-						       ExportStyleDlgProc);
-  if (DialogBox (hdllInstance,
-		 (ls.use_printer == 0) ? "ExportStyleDlgBox"
-		 : "ExportPrintDlgBox",
-		 ScilabGC->hWndParent, lpfnExportStyleDlgProc) == IDOK)
-    {
-      status = TRUE;
-    }
-  return status;
-}
-/*-----------------------------------------------------------------------------------*/
-static void SavePs (struct BCG *ScilabGC)
-{
-  
-
-  char *d, ori;
-  BYTE *s;
-  char str[] = "[SAVESCG]XScilab Postscript[EOS]*[EOS]";
-  int flag, ierr = 0;
-  /** getting ls flags **/
-  ls.use_printer = 0;
-  if (ExportStyle (ScilabGC) == FALSE)
-    return;
-  /** getting filename **/
-  d = filename;
-  TranslateMacro (str);
-  s = str;
-  flag = SciOpenSave (ScilabGC->hWndParent, &s,TRUE,&d, &ierr);
-  if (flag == 0 || ierr == 1)
-    {
-      return;
-    }
-  *d = '\0';
-
-  
-
-  switch (ls.ps_type)
-    {
-    case 0:
-      /** postscript Epsf file **/
-      SetCursor (LoadCursor (NULL, IDC_WAIT));
-      wininfo ("Epsf file generation");
-      dos2win32 (filename, filename1);
-      scig_tops (ScilabGC->CurWindow, ls.colored, filename1, "Pos");
-      ori = (ls.land == 1) ? 'l' : 'p';
-      ScilabPsToEps (ori, filename1, filename);
-      wininfo ("end of Epsf generation");
-      SetCursor (LoadCursor (NULL, IDC_CROSS));
-      break;
-    case 1:
-      SetCursor (LoadCursor (NULL, IDC_WAIT));
-      scig_tops (ScilabGC->CurWindow, ls.colored, filename, "Pos");
-      wininfo ("end of Ps file generation");
-      SetCursor (LoadCursor (NULL, IDC_CROSS));
-      break;
-    case 2:
-      /** Epsf + Tex file **/
-      SetCursor (LoadCursor (NULL, IDC_WAIT));
-      wininfo ("Epsf and LaTeX files generation");
-      dos2win32 (filename, filename1);
-      scig_tops (ScilabGC->CurWindow, ls.colored, filename1, "Pos");
-      ori = (ls.land == 1) ? 'l' : 'p';
-      ScilabPsToTeX (ori, filename1, filename, 1.0, 1.0);
-      wininfo ("end of Epsf file and LaTeX file generation");
-      SetCursor (LoadCursor (NULL, IDC_CROSS));
-      break;
-    case 3:
-      SetCursor (LoadCursor (NULL, IDC_WAIT));
-      scig_tops (ScilabGC->CurWindow, ls.colored, filename, "Fig");
-      wininfo ("end of Xfig file generation");
-      SetCursor (LoadCursor (NULL, IDC_CROSS));
-      break;
-    case 4:
-      SetCursor (LoadCursor (NULL, IDC_WAIT));
-      scig_tops (ScilabGC->CurWindow, ls.colored, filename, "GIF");
-      wininfo ("end of GIF file generation");
-      SetCursor (LoadCursor (NULL, IDC_CROSS));
-      break;
-    case 5:
-      SetCursor (LoadCursor (NULL, IDC_WAIT));
-      scig_tops (ScilabGC->CurWindow, ls.colored, filename, "PPM");
-      wininfo ("end of PPM file generation");
-      SetCursor (LoadCursor (NULL, IDC_CROSS));
-      break;
-    case 6:
-      SetCursor (LoadCursor (NULL, IDC_WAIT));
-      ExportBMP(ScilabGC,filename);
-      wininfo ("end of BMP file generation");
-      SetCursor (LoadCursor (NULL, IDC_CROSS));
-      break;
-    case 7:
-      SetCursor (LoadCursor (NULL, IDC_WAIT));
-      ExportEMF(ScilabGC,filename);
-      wininfo ("end of EMF file generation");
-      SetCursor (LoadCursor (NULL, IDC_CROSS));
-      break;
-    }
-	
-	
-}
-/*-----------------------------------------------------------------------------------*/
-static void dos2win32 (char *filename, char *filename1)
-{
-#ifdef CVT_PATH_BEG
-  if (filename[1] == ':')
-    {
-      *filename1++ = '/';
-      *filename1++ = '/';
-      *filename1++ = *filename++;
-      filename++;
-    }
-#endif
-  while (*filename != '\0')
-    {
-      *filename1++ = *filename++;
-      if (*(filename1 - 1) == '\\')
-	*(filename1 - 1) = '/';
-    }
-  *filename1 = '\0';
-}
-/*-----------------------------------------------------------------------------------*/
-static void PrintPs (struct BCG *ScilabGC)
-{
-  char *p1;
-  char ori;
-  /** getting ls flags **/
-  ls.use_printer = 1;
-  if (ExportStyle (ScilabGC) == FALSE)
-    return;
-  /** getting filename **/
-  if ((p1 = getenv ("TMPDIR")) == (char *) 0)
-    {
-      sciprint ("Cannot find environment variable TMPDIR\r\n");
-      return;
-    }
-  sprintf (filename, "%s/scilab-%d", p1, (int) ScilabGC->CurWindow);
-  /** sciprint(" file name [%s] color=%d\r\n",filename,ls.colored); **/
-  dos2win32 (filename, filename1);
-  scig_tops (ScilabGC->CurWindow, ls.colored, filename1, "Pos");
-  ori = (ls.land == 1) ? 'l' : 'p';
-  if (ScilabPsToEps (ori, filename1, filename) == 0)
-    {
-      if (gp_printfile (hdllInstance, ScilabGC->hWndParent, filename,
-			(char *) 0) == FALSE)
-	sciprint ("Error while printing\r\n");
-    }
-  /** filename is destroyed when we quit scilab **/
-}
-/*-----------------------------------------------------------------------------------*/
 /* used by command_handler in metanet */
-
 static void scig_command_scilabgc (int number, void f (struct BCG *))
 {
   struct BCG *ScilabGC = GetWindowXgcNumber (number);
@@ -1043,19 +896,14 @@ static void scig_command_scilabgc (int number, void f (struct BCG *))
     f (ScilabGC);
 }
 /*-----------------------------------------------------------------------------------*/
-void scig_h_winmeth_print (integer number)
-{
-  scig_command_scilabgc (number,CopyPrint);
-}
-/*-----------------------------------------------------------------------------------*/
 void scig_h_copyclip (integer number)
 {
-  scig_command_scilabgc (number, NewCopyClip);
+  scig_command_scilabgc (number, CopyToClipboardEMF);
 }
 /*-----------------------------------------------------------------------------------*/
 void scig_h_copyclip1 (integer number)
 {
-  scig_command_scilabgc (number, CopyClip);
+  scig_command_scilabgc (number, CopyToClipboardBitmap);
 }
 /*-----------------------------------------------------------------------------------*/
 void scig_print (integer number)
@@ -1072,13 +920,9 @@ void UpdateFileGraphNameMenu(struct BCG *ScilabGC)
 {
 #define FILEGRAPHMENUFRENCH "wgscilabF.mnu"
 #define FILEGRAPHMENUENGLISH "wgscilabE.mnu"
-	
-  char szModuleName[MAX_PATH];
-  LPSTR tail;
-	
-  HINSTANCE hInstance=NULL;
   extern char ScilexWindowName[MAX_PATH];
-  LPTW lptw;
+  char *ScilabDirectory=NULL;
+
   HWND hWndTmpScilex=FindWindow(NULL,ScilexWindowName);
   int LangCode=0; /*English*/
 
@@ -1086,7 +930,7 @@ void UpdateFileGraphNameMenu(struct BCG *ScilabGC)
     {
       if (hWndTmpScilex)
 	{
-	  lptw = (LPTW) GetWindowLong (FindWindow(NULL,ScilexWindowName), 0);
+	  LPTW lptw=GetTextWinScilab();
 	  LangCode=lptw->lpmw->CodeLanguage;
 	}
       else LangCode=0;
@@ -1094,32 +938,22 @@ void UpdateFileGraphNameMenu(struct BCG *ScilabGC)
   else LangCode=0;
   ScilabGC->lpmw.CodeLanguage=LangCode;
 
-  hInstance=(HINSTANCE) GetModuleHandle("LibScilab");   		
+  ScilabDirectory=GetScilabDirectory(FALSE);
 	
-	
-  GetModuleFileName (hInstance,szModuleName, MAX_PATH);
-	
-  if ((tail = strrchr (szModuleName, '\\')) != (LPSTR) NULL)
-    {
-      tail++;
-      *tail = '\0';
-    }
-	
-  if (ScilabGC->lpgw->szMenuName!=NULL) free(ScilabGC->lpgw->szMenuName);
-  ScilabGC->lpgw->szMenuName = (LPSTR) malloc (strlen (szModuleName) + strlen (FILEGRAPHMENUENGLISH) + 1);
-  strcpy (ScilabGC->lpgw->szMenuName, szModuleName);
-
+  if (ScilabGC->lpgw->szMenuName!=NULL) FREE(ScilabGC->lpgw->szMenuName);
+  ScilabGC->lpgw->szMenuName = (LPSTR) MALLOC (strlen (ScilabDirectory) +strlen("\\bin\\")+ strlen (FILEGRAPHMENUENGLISH) + 1);
 	
   switch (LangCode)
-    {
+  {
     case 1:
-      strcat (ScilabGC->lpgw->szMenuName, FILEGRAPHMENUFRENCH);
+	  wsprintf(ScilabGC->lpgw->szMenuName,"%s\\bin\\%s",ScilabDirectory, FILEGRAPHMENUFRENCH);
       break;
     default : case 0:
-      strcat (ScilabGC->lpgw->szMenuName, FILEGRAPHMENUENGLISH);
+	  wsprintf(ScilabGC->lpgw->szMenuName,"%s\\bin\\%s",ScilabDirectory, FILEGRAPHMENUENGLISH);
       break;
-    }
-	
+  }
+
+  if (ScilabDirectory){FREE(ScilabDirectory);ScilabDirectory=NULL;}	
 }
 /*-----------------------------------------------------------------------------------*/
 void NewFigure(struct BCG * ScilabGC)
@@ -1165,7 +999,7 @@ int FindFreeGraphicWindow(struct BCG * ScilabGC)
       C2F(getwins)(&num,&ids ,&iflag);
       sizetab=num;
 
-      tab=(integer*)malloc(sizeof(integer)*sizetab);
+      tab=(integer*)MALLOC(sizeof(integer)*sizetab);
       for(i=0;i<sizetab;i++) tab[i]=0;
 
       iflag = 1; 
@@ -1176,8 +1010,193 @@ int FindFreeGraphicWindow(struct BCG * ScilabGC)
 	  if(FreeNumber<tab[i]) FreeNumber=tab[i];	
 	}
       FreeNumber=FreeNumber+1;
-      free(tab);
+      FREE(tab);
     } 
   return FreeNumber;
 }
 /*-----------------------------------------------------------------------------------*/
+void RefreshMenus(struct BCG * ScilabGC)
+{
+	//if (ScilabGC->graphicsversion!=0) 
+	//{
+	//	BOOL LockToolBar=ScilabGC->lpmw.LockToolBar;
+	//	int nButton=ScilabGC->lpmw.nButton;
+	//	BOOL StateToolBar=ScilabGC->lpmw.ShowToolBar;
+
+	//	DestroyMenu(ScilabGC->hMenuRoot);
+	//	ScilabGC->hMenuRoot=NULL;
+	//	ScilabGC->IDM_Count=1;
+	//	SetMenu(ScilabGC->hWndParent,NULL);
+
+	//	UpdateFileGraphNameMenu( ScilabGC);
+	//	LoadGraphMacros( ScilabGC);
+
+	//	ScilabGC->lpmw.nButton=nButton;
+	//	ScilabGC->lpmw.ShowToolBar=StateToolBar;
+	//	ScilabGC->lpmw.LockToolBar=LockToolBar;
+
+	//}
+	//else
+	//{
+	//	BOOL LockToolBar=ScilabGC->lpmw.LockToolBar;
+	//	int nButton=ScilabGC->lpmw.nButton;
+	//	BOOL StateToolBar=ScilabGC->lpmw.ShowToolBar;
+
+
+	//	DestroyMenu(GetMenu(ScilabGC->hWndParent));
+	//	SetMenu(ScilabGC->hWndParent,NULL);
+	//	CloseGraphMacros ( ScilabGC);
+	//	ScilabGC->lpmw.nButton=nButton;
+	//	ScilabGC->lpmw.ShowToolBar=StateToolBar;
+	//	ScilabGC->lpmw.LockToolBar=LockToolBar;
+
+	//	ScilabGC->hMenuRoot=CreateMenu();
+	//	ScilabGC->IDM_Count=1;
+
+	//	SetMenu(ScilabGC->hWndParent,ScilabGC->hMenuRoot);
+
+	//}
+	int WinNum=ScilabGC->CurWindow;
+	BOOL LockToolBar=ScilabGC->lpmw.LockToolBar;
+	int nButton=ScilabGC->lpmw.nButton;
+	BOOL StateToolBar=ScilabGC->lpmw.ShowToolBar;
+
+	DestroyMenu(GetMenu(ScilabGC->hWndParent));
+	ScilabGC->hMenuRoot=NULL;
+	ScilabGC->IDM_Count=1;
+	SetMenu(ScilabGC->hWndParent,NULL);
+
+	CloseGraphMacros (ScilabGC);
+	CreateGedMenus(ScilabGC);
+
+	ScilabGC->lpmw.nButton=nButton;
+	ScilabGC->lpmw.ShowToolBar=StateToolBar;
+	ScilabGC->lpmw.LockToolBar=LockToolBar;
+
+}
+/*-----------------------------------------------------------------------------------*/
+void CreateGedMenus(struct BCG * ScilabGC)
+{
+	int WinNum=ScilabGC->CurWindow;
+	if (ScilabGC->graphicsversion!=0)
+	{
+		integer ne=3, menutyp=2, ierr;
+		char *EditMenusE[]={"&Select figure","&Redraw figure","&Erase figure"};
+		char *EditMenusF[]={"&Selectionner figure","&Redessiner figure","&Effacer figure"};
+
+		UpdateFileGraphNameMenu(ScilabGC);
+		LoadGraphMacros(ScilabGC);
+
+		switch( ScilabGC->lpmw.CodeLanguage)
+		{
+			case 1:
+				AddMenu(&WinNum,"&Editer", EditMenusF, &ne, &menutyp, "ged", &ierr);
+			break;
+			default:
+				AddMenu(&WinNum,"&Edit", EditMenusE, &ne, &menutyp, "ged", &ierr);
+			break;
+		}
+	}
+	else
+	{
+		/*
+		for new menus
+		ScilabXgc->hMenuRoot=CreateMenu();
+		ScilabXgc->IDM_Count=1;
+
+		SetMenu(ScilabXgc->hWndParent,ScilabXgc->hMenuRoot); 
+		*/
+	#ifdef WITH_TK
+		integer ne=9, menutyp=2, ierr;
+		char *EditMenusE[]={"&Select figure as current","&Redraw figure","&Erase figure","[--]","Figure properties","Current &axes properties","[--]",MSG_SCIMSG116,MSG_SCIMSG117};
+		char *EditMenusF[]={"&Selectionner figure comme courante","&Redessiner figure","&Effacer figure","[--]","Proprits de la &figure","Proprits des &axes courants","[--]",MSG_SCIMSG118,MSG_SCIMSG119};
+	#else
+		integer ne=3, menutyp=2, ierr;
+		char *EditMenusE[]={"&Select figure","&Redraw figure","&Erase figure"};
+		char *EditMenusF[]={"&Selectionner figure","&Redessiner figure","&Effacer figure"};
+	#endif
+
+		UpdateFileGraphNameMenu(ScilabGC);
+		LoadGraphMacros(ScilabGC);
+
+		switch( ScilabGC->lpmw.CodeLanguage)
+		{
+			case 1:
+				AddMenu(&WinNum,"&Editer", EditMenusF, &ne, &menutyp, "ged", &ierr);
+			break;
+
+			default:
+				AddMenu(&WinNum,"&Edit", EditMenusE, &ne, &menutyp, "ged", &ierr);
+			break;
+		}
+
+	}
+}
+/*-----------------------------------------------------------------------------------*/
+BOOL SendMacroEntityPicker(struct BCG * ScilabGC,int id)
+{
+	BOOL bOK=FALSE;
+	char command[1024];
+	
+	if (IsEntityPickerMenu(ScilabGC,id))
+	{
+		if (id == 19)
+		{
+			wsprintf(command,"ged(6,%d);",ScilabGC->CurWindow); /* Start */
+			ModifyEntityPickerToolbar(ScilabGC,TRUE);
+			StoreCommand(command);
+			bOK=TRUE;
+		}
+
+		if (id == 20)
+		{
+			wsprintf(command,"ged(7,%d);",ScilabGC->CurWindow); /* Stop */
+			ModifyEntityPickerToolbar(ScilabGC,FALSE);
+			StoreCommand(command);
+			bOK=TRUE;
+		}
+	}
+	
+	return bOK;
+}
+/*-----------------------------------------------------------------------------------*/
+BOOL IsEntityPickerMenu(struct BCG * ScilabGC,int id)
+{
+	BOOL bOK=FALSE;
+	if (id == 19)
+	{
+		#define lenStringMenu 64
+		char CurrentStringMenu[lenStringMenu];
+		GetMenuString(ScilabGC->lpmw.hMenu,id,CurrentStringMenu,lenStringMenu-1,MF_BYCOMMAND);
+		switch (ScilabGC->lpmw.CodeLanguage)
+		{
+			case 1:
+				if (strcmp(CurrentStringMenu,MSG_SCIMSG118)==0) bOK=TRUE;
+			break;
+
+			case 0: default:
+				if (strcmp(CurrentStringMenu,MSG_SCIMSG116)==0) bOK=TRUE;
+			break;
+		}
+	}
+
+	if (id == 20)
+	{
+		#define lenStringMenu 64
+		char CurrentStringMenu[lenStringMenu];
+		GetMenuString(ScilabGC->lpmw.hMenu,id,CurrentStringMenu,lenStringMenu-1,MF_BYCOMMAND);
+		switch (ScilabGC->lpmw.CodeLanguage)
+		{
+			case 1:
+				if (strcmp(CurrentStringMenu,MSG_SCIMSG119)==0) bOK=TRUE;
+			break;
+
+			case 0: default:
+				if (strcmp(CurrentStringMenu,MSG_SCIMSG117)==0) bOK=TRUE;
+			break;
+		}
+	}
+
+	return bOK;
+}
+/*-----------------------------------------------------------------------------------*/
