@@ -5,6 +5,52 @@
 #
 # -->   set.tcl
 
+proc GetGoodValue { h val} {
+    global Win
+    global STRING_$h
+    
+    set mystring STRING_$h
+    if {[array exists $mystring] == 1} {
+  	set size [array size $mystring]
+  	set tmp ""
+	set tab $mystring
+	
+	set sizlist [array size $mystring]
+        set sortedlist [lsort [array names $mystring]]
+	
+	set index [expr $sizlist-1]
+	set aa [lindex $sortedlist $index]
+
+	set aa [split $aa ',']
+	
+	set xsize [lindex $aa 0]
+	set ysize [lindex $aa 1]
+	
+	if { $xsize == 1} {
+	    for {set i 1} {$i<=$size} {incr i} { 
+		set aa $mystring\(1,$i\)
+		set bb $$aa
+		lappend tmp [expr $bb]
+	    }
+	    set tmp [join $tmp "|"]
+	    return $tmp
+	} elseif {  $ysize == 1} {
+	    for {set i 1} {$i<=$size} {incr i} { 
+		set aa $mystring\($i,1\)
+		set bb $$aa
+		lappend tmp [expr $bb]
+	    }
+	    set tmp [join $tmp "|"]
+	    return $tmp
+	} else {
+	    ScilabEval "disp(\'wrong input : a matrix has been entered\')";
+	    return $val
+	}
+    } else {
+	return $val
+    }
+}
+
 
 ######################################################################################
 proc SetField { handle field value } {
@@ -19,6 +65,10 @@ proc SetField { handle field value } {
     set name $Win($handle);
     global "$name";
     
+    if { $field == "string" } {
+	set value [GetGoodValue $handle $value]
+    }
+    
     set control [set "$name\(control)"];
     set handle [GetRealParent $handle $control];
     set style [set "$name\(style)"];
@@ -61,15 +111,15 @@ proc SetField { handle field value } {
 	    # implemented fields
 	    set KnownFunc  { backgroundcolor callback fontangle fontname fontsize \
 		    fontweight fontunits foregroundcolor horizontalalignment listboxtop max min \
-		    parent position string units value figure_name verticalalignment };
-	    
+		    parent position string sliderstep units value figure_name verticalalignment };
+
 	    # is it an implemented?
 	    set idx  [lsearch $KnownFunc $field];
 	    
 	    if { $idx != -1} {
 		# YES there is a specific behaviour
 		# call the specific setfield
-		"Set$field" $name $value;    
+		"Set$field" $name $value;    # param = (name, str) dans Setstring 
 		
 		set path [set "$name\(path)"];
 		bind $path <Configure> {ChgConfigure %W %w %h};
@@ -286,6 +336,7 @@ proc Setcallback { name callback } {
 	
 	edit {$path configure -command $command;}
 	pushbutton {$path configure -command $command;}
+	radiobutton  {$path configure -command $command;}
 	checkbox  {$path configure -command $command;}
 	slider {
 	    set command "SliderSciCallback $name $callback";
@@ -405,6 +456,7 @@ proc Sethorizontalalignment { name value } {
      switch -exact -- $style {
 	text {$path configure -anchor $anchor}
 	edit {$path configure -justify $value}
+	radiobutton {$path configure -anchor $anchor}
 	checkbox {$path configure -anchor $anchor}
 	#listbox {$path.list configure -anchor $AnchorEq($value)}
     }
@@ -447,6 +499,7 @@ proc Setverticalalignment { name value } {
     switch -exact -- $style {
 	text {$path configure -anchor $anchor}
 	edit { unset "$name\(verticalalignment)"}
+	radiobutton {$path configure -anchor $anchor}
 	checkbox {$path configure -anchor $anchor}
 	#listbox {$path.list configure -anchor $AnchorEq($value)}
 
@@ -484,6 +537,7 @@ proc Setmax { name  value} {
   set style [set "$name\(style)"];
 
   switch -exact -- $style {
+    "radiobutton" { $path configure -value $value }
     "checkbox" { $path configure -onvalue $value }
     "slider" { $path configure -to $value }
     "listbox" {
@@ -515,6 +569,7 @@ proc Setmin { name  value} {
   set style [set "$name\(style)"];
 
   switch -exact -- $style {
+    "radiobutton" { $path configure -value $value }
     "checkbox" { $path configure -offvalue $value }
     "slider" { $path configure -from $value }
     "listbox" {
@@ -725,12 +780,18 @@ proc Setsliderstep { name value } {
     set path [set "$name\(path)"];
 
     set  step [split $value "|"];
-    set smallstep  [set $step(1)];
-    set bigstep [set $step(2)];
-
-    
-
+    set smallstep  [lindex $step 0];
+    set bigstep [lindex $step 1];
 
+#enrico
+    if {$bigstep==""} {set bigstep "0"}
+    switch -exact -- $style {
+       slider {
+          $path configure -resolution $smallstep -bigincrement $bigstep
+       }
+	default {
+	}
+    }
 }
 ######################################################################################
 proc Setstring { name  str} {
@@ -760,6 +821,7 @@ proc Setstring { name  str} {
 	    $path configure -text [lindex $item 0]
 	}
 	text { $path configure -text $str; }
+	radiobutton { $path configure -text $str; }
 	checkbox { $path configure -text $str; }
 	pushbutton { $path configure -text $str;}
 	default {
